# Generated by roxygen2: do not edit by hand

export(EM_FMM_SemiSupervised)
export(EM_FMM_SemiSupervised_Complete_Initial)
export(EM_FMM_SemiSupervised_Initial)
export(bayesclassifier)
export(compute_d2)
export(error_beta_classification)
export(get_clusterprobs)
export(get_entropy)
export(initialestimate)
export(logsumexp)
export(neg_loglik)
export(normalise_logprob)
export(pack_theta)
export(rmix)
export(simulate_mixed_missingness)
export(unpack_theta)
importFrom(matrixStats,rowLogSumExps)
importFrom(mvtnorm,dmvnorm)
importFrom(mvtnorm,rmvnorm)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,glm)
importFrom(stats,plogis)
importFrom(stats,rbinom)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
